@@ -1,17 +1,18 @@
 import ctypes
+import math
 import sys
 import threading
 import time
-import math
 from functools import reduce
 from typing import List, Optional, Callable
-import os
+
 import apng
+import qimage2ndarray as q2n
 from PyQt5 import QtWidgets
 
-from .consts import *
 from ._utils import invoke_in_app_thread
-from .graphwin import GraphWin
+from .consts import *
+from .graphwin import GraphWin, KeyMessage, MouseMessage
 from .image import Image
 from .utils3d import *
 
@@ -30,32 +31,36 @@ __all__ = [
     'reflect', 'flip', 'mirror', 'reset_transform', 'save_settings', 'restore_settings',
     'get_width', 'get_height', 'get_write_mode', 'set_write_mode', 'get_transform', 'set_transform',
     'push_transform', 'pop_transform', 'set_rect_mode', 'get_rect_mode', 'set_ellipse_mode', 'get_ellipse_mode',
+    'set_antialiasing',
     # drawing functions #
     'draw_point', 'put_pixel', 'get_pixel', 'line', 'draw_line', 'move_to', 'move_rel', 'line_to', 'line_rel',
     'circle', 'draw_circle', 'fill_circle', 'ellipse', 'draw_ellipse', 'fill_ellipse',
     'arc', 'draw_arc', 'pie', 'draw_pie', 'fill_pie', 'chord', 'draw_chord', 'fill_chord',
     'bezier', 'draw_bezier', 'lines', 'draw_lines', 'poly_line', 'draw_poly_line', 'polygon', 'draw_polygon',
     'fill_polygon', 'rect', 'draw_rect', 'fill_rect', 'rounded_rect', 'draw_rounded_rect', 'fill_rounded_rect',
-    'flood_fill', 'draw_image', 'capture_screen', 'clear_device', 'clear_view_port',
+    'flood_fill', 'draw_image', 'clear_device', 'clear_view_port',
     'quadratic', 'draw_quadratic', 'fill_image', 'clear', 'draw_curve', 'curve',
     'begin_shape', 'end_shape', 'vertex', 'bezier_vertex', 'quadratic_vertex', 'curve_vertex',
+    'bezier_point','bezier_tangent','curve_point','curve_tangent',
     # text functions #
     'draw_text', 'draw_rect_text', 'text_width', 'text_height',
     # image functions #
-    'set_target', 'get_target', 'create_image', 'save_image', 'close_image', 'load_image', 'put_image',
+    'set_target', 'get_target', 'create_image','create_image_from_ndarray', 'save_image', 'close_image', 'load_image', 'put_image',
+    "create_image_from_file",'capture_screen', 'get_image',
     # time control functions#
     'pause', 'delay', 'delay_fps', 'delay_jfps', 'is_run',
     # keyboard and mouse functions #
     'has_kb_msg', 'has_kb_hit', 'has_mouse_msg', 'get_key', 'get_char', 'get_mouse_msg', 'get_cursor_pos', 'get_click',
-    "contains_left_button", "contains_right_button", "contains_mid_button",
+    "contains_left_button", "contains_right_button", "contains_mid_button","clear_key_msg","clear_char_msg","clear_mouse_msg",
+    "contains_alt", "contains_ctrl", "contains_meta", "contains_shift",
     # init and close graph window #
     'init_graph', 'close_graph', 'set_caption', 'get_graphics_window', 'show_image',
     # animation
     'begin_recording', 'save_recording', 'add_record', 'end_recording',
     # utility functions #
-    'color_gray', 'color_rgb', 'color_cmyk', 'color_hsv', 'rgb', 'to_alpha', 'pol2cart', 'cart2pol',
+    'color_gray', 'color_rgb', 'color_cmyk', 'color_hsv', 'color_hsl','rgb', 'to_alpha', 'pol2cart', 'cart2pol',
     # utility functions for 3d
-    'ortho_look_at', 'isometric_projection', 'cart2spher', 'spher2cart',
+    'ortho_look_at', 'isometric_projection', 'cart2sphere', 'sphere2cart',
     # 'GraphWin',
     'Image',
     # Easy run mode
@@ -397,6 +402,18 @@ def get_ellipse_mode(image: Image = None) -> int:
     return image.get_ellipse_mode()
 
 
+def set_antialiasing(anti:bool=True, image: Image = None)->None:
+    """
+    Set Anti Aliasing
+
+    :param anti: if antialiasing should be set
+    :param image: the image to set anti aliasing. None means it is the target image
+        (see set_target() and get_target()).
+    """
+    image = _get_target_image(image)
+    image.set_antialiasing(anti)
+
+
 def get_transform(image: Image = None) -> QtGui.QTransform:
     """
     Get transform matrix of the image.
@@ -1577,24 +1594,23 @@ def draw_image(x: int, y: int, src_image: Image, src_x: int = 0, src_y: int = 0,
     dst_image = _get_target_image(dst_image)
     dst_image.draw_image(x, y, src_image, src_x, src_y, src_width, src_height, with_background, composition_mode)
 
-
 put_image = draw_image
 
-
-def capture_screen(left: int, top: int, right: int, bottom: int, target_img: Image):
+def capture_screen(left: int, top: int, width: int, height: int, target_img: Image):
     """
-    Caputre specified region on the graphics windows to target image.
+    Capture specified region on the graphics windows to target image.
 
     :param left: x coordinate of the capture region\'s upper left corner
     :param top: y coordinate of the capture region\'s upper left corner
-    :param right: x coordinate of the capture region\'s bottom right corner
-    :param bottom: y coordinate of the capture region\'s bottom right corner
+    :param width: width of the capture region
+    :param height: height of the capture region
     :param target_img: image to save the capture
     """
     _check_not_headless_and_in_shell()
     draw_image(0, 0, _win.get_canvas(), src_x=left, src_y=top,
-               src_width=right - left, src_height=bottom - top, dst_image=target_img)
+               src_width=width, src_height=height, dst_image=target_img)
 
+get_image = capture_screen
 
 def clear_device(image: Image = None):
     """
@@ -1633,6 +1649,87 @@ def clear_view_port(image: Image = None):
     image.clear_view_port()
 
 
+def bezier_point(p0:float, p1:float, p2:float, p3:float, t:float):
+    """
+    Calculate the position of a point with parameter t on the cubic bezier curve.
+
+    Note that the parameter t must >=0 and <=1 .
+
+    :param p0: position of the first control point
+    :param p1: position of the second control point
+    :param p2: position of the third control point
+    :param p3: position of the forth control point
+    :param t: the parameter t
+    :return: position of the point
+    """
+    if t<0 or t>1:
+        raise ValueError("t must in [0..1]")
+    return (1-t) ** 3 * p0 + 3 * (1 - t) ** 2 * t * p1 + 3 * (1 - t) * t ** 2 * p2 + t ** 3 * p3
+
+def bezier_tangent(p0:float, p1:float, p2:float, p3:float, t:float):
+    """
+    Calculate the tangent of the point with parameter t on the cubic bezier curve.
+
+    Note that the parameter t must >=0 and <=1 .
+
+    :param p0: position of the first control point
+    :param p1: position of the second control point
+    :param p2: position of the third control point
+    :param p3: position of the forth control point
+    :param t: the parameter t
+    :return: tangent of the point
+    """
+    if t<0 or t>1:
+        raise ValueError("t must in [0..1]")
+    return 3*(1-t)**2*(p1-p0)+6*(1-t)*t*(p2-p1)+3*t**2*(p3-p2)
+
+
+def curve_point(p0:float, p1:float, p2:float, p3:float, t:float):
+    """
+    Calculate the position of the point with parameter t on a Catmull-Rom spline.
+
+    Note that the parameter t must >=0 and <=1 .
+
+    :param p0: position of the first control point
+    :param p1: position of the second control point
+    :param p2: position of the third control point
+    :param p3: position of the forth control point
+    :param t: the parameter t
+    :return: position of the point
+    """
+    if t<0 or t>1:
+        raise ValueError("t must in [0..1]")
+
+    x0 = p1
+    x1 = -p0 / 6 + p1 + p2 / 6
+    x2 = p1 / 6 + p2 - p3 / 6
+    x3 = p2
+
+    return bezier_point(x0,x1,x2,x3,t)
+
+def curve_tangent(p0:float, p1:float, p2:float, p3:float, t:float):
+    """
+    Calculate the tangent of the point with parameter t on a Catmull-Rom spline.
+
+    Note that the parameter t must >=0 and <=1 .
+
+    :param p0: position of the first control point
+    :param p1: position of the second control point
+    :param p2: position of the third control point
+    :param p3: position of the forth control point
+    :param t: the parameter t
+    :return: tangent of the point
+    """
+    if t<0 or t>1:
+        raise ValueError("t must in [0..1]")
+
+    x0 = p1
+    x1 = -p0 / 6 + p1 + p2 / 6
+    x2 = p1 / 6 + p2 - p3 / 6
+    x3 = p2
+
+    return bezier_tangent(x0,x1,x2,x3,t)
+
 def begin_shape(type=VertexType.POLY_LINE, image: Image = None):
     """
     Begin a shape definition
@@ -1813,6 +1910,11 @@ def create_image(width, height) -> Image:
     """
     return Image.create(width, height)
 
+def create_image_from_ndarray(array) -> Image:
+    """
+    Convert a ndarray (opencv 3.0 image) to an Image object
+    """
+    return Image(q2n.array2qimage(array))
 
 def close_image(image: Image):
     """
@@ -1863,15 +1965,7 @@ def show_image(image: Image = None):
     image.display_in_ipython()
 
 
-def create_image_from_file(filename: str) -> Image:
-    """
-    Load image form the specified file.
-
-    :param filename: path of the file
-    :return: loaded image
-    """
-    image = QtGui.QImage(filename)
-    return Image(image)
+create_image_from_file = load_image
 
 
 # utils
@@ -1903,6 +1997,7 @@ def color_rgb(red: int, green: int, blue: int, alpha: int = 255) -> QtGui.QColor
 rgb = color_rgb
 
 
+
 def color_cmyk(c: int, m: int, y: int, k: int, alpha: int = 255) -> QtGui.QColor:
     """
     Create a color with CMYK color values c,m,y,k.
@@ -1916,6 +2011,18 @@ def color_cmyk(c: int, m: int, y: int, k: int, alpha: int = 255) -> QtGui.QColor
     """
     return QtGui.QColor.fromCmyk(c, m, y, k, alpha)
 
+def color_hsl(h: int, s: int, l: int, alpha: int = 255) -> QtGui.QColor:
+    """
+    Create a color with HSL color values h,s,l.
+
+    :param h: hue value
+    :param s: saturation value
+    :param l: lightness value
+    :param alpha: alpha channel value of the color. 255 means fully opaque
+    :return: the color
+    """
+    return QtGui.QColor.fromHsl(h, s, l, alpha)
+
 
 def color_hsv(h: int, s: int, v: int, alpha: int = 255) -> QtGui.QColor:
     """
@@ -1923,7 +2030,7 @@ def color_hsv(h: int, s: int, v: int, alpha: int = 255) -> QtGui.QColor:
 
     :param h: hue value
     :param s: saturation value
-    :param v: lightness value
+    :param v: Value
     :param alpha: alpha channel value of the color. 255 means fully opaque
     :return: the color
     """
@@ -2055,7 +2162,7 @@ def delay_jfps(fps, max_skip_count=0):
 
 def has_kb_hit() -> bool:
     """
-    See if any ascii char key is hitted in the last 100 ms.
+    See if any ascii char key hit message in the message queue.
 
     Use it with get_char().
 
@@ -2067,7 +2174,7 @@ def has_kb_hit() -> bool:
 
 def has_kb_msg() -> bool:
     """
-    See if any key is hit in the last 100 ms.
+    See if any key hit message in the message queue.
 
     Use it with get_key().
 
@@ -2079,7 +2186,7 @@ def has_kb_msg() -> bool:
 
 def has_mouse_msg() -> bool:
     """
-    See if there is any mouse message(event) in the last 100 ms.
+    See if there is any mouse message(event) in the message queue.
 
     Use it with get_mouse_msg().
 
@@ -2089,38 +2196,88 @@ def has_mouse_msg() -> bool:
     return _win.has_mouse_msg()
 
 
-def get_mouse_msg() -> (int, int, int, int):
+def get_mouse_msg() -> MouseMessage:
     """
     Get the mouse message.
 
-    If there is not any  mouse button is pressed or released in last 100 ms, the program will stop and wait for
-    the next mouse message.
+    If there is not any  mouse message, the program will stop and wait for it.
 
-    :return: x of the cursor, y of the cursor , type, mouse buttons down
-        ( QtCore.Qt.LeftButton or QtCore.Qt.RightButton or QtCore.Qt.MidButton or QtCore.Qt.NoButton)
+    :return: mouse message
     """
     _check_not_headless_and_in_shell()
     return _win.get_mouse_msg()
 
 
-def get_click() -> (int, int, int):
+def get_click() -> MouseMessage:
     """
     Get the mouse click message.
 
-    If there is not any  mouse button is clicked in last 100 ms, the program will stop and wait for
-    the next clicking.
+    If there isn't any clicked event, the program will stop and wait for it.
 
-    :return: x of the cursor, y of the cursor , mouse buttons down
-        ( QtCore.Qt.LeftButton or QtCore.Qt.RightButton or QtCore.Qt.MidButton or QtCore.Qt.NoButton)
+    :return: mouse message
     """
     _check_not_headless_and_in_shell()
-    while is_run():
-        x, y, _type, buttons = _win.get_mouse_msg()
-        if _type == MouseMessageType.RELEASE_MESSAGE:
-            return x, y, buttons
+    return _win.get_click()
 
-    return 0, 0, QtCore.Qt.NoButton
 
+def clear_key_msg():
+    """
+    Clear all keyboard hit messages.
+    """
+    _check_not_headless_and_in_shell()
+    _win.clear_key_msg()
+
+
+def clear_char_msg(self):
+    """
+    Clear all char key hit messages.
+    """
+    _check_not_headless_and_in_shell()
+    _win.clear_char_msg()
+
+
+def clear_mouse_msg(self):
+    """
+    Clear all mouse messages.
+    """
+    _check_not_headless_and_in_shell()
+    _win.clear_mouse_msg()
+
+def contains_ctrl(modifiers)->bool:
+    """
+    Test if the modifiers contains the control key
+
+    :param modifiers: the modifiers to be tested
+    :return: if the modifiers contains the control key
+    """
+    return modifiers & QtCore.Qt.ControlModifier
+
+def contains_shift(modifiers)->bool:
+    """
+    Test if the modifiers contains the shift key
+
+    :param modifiers: the modifiers to be tested
+    :return: if the modifiers contains the shift key
+    """
+    return modifiers & QtCore.Qt.ShiftModifier
+
+def contains_alt(modifiers)->bool:
+    """
+    Test if the modifiers contains the alt key
+
+    :param modifiers: the modifiers to be tested
+    :return: if the modifiers contains the alt key
+    """
+    return modifiers & QtCore.Qt.AltModifier
+
+def contains_meta(modifiers)->bool:
+    """
+    Test if the modifiers contains the meta key
+
+    :param modifiers: the modifiers to be tested
+    :return: if the modifiers contains the meta key
+    """
+    return modifiers & QtCore.Qt.MetaModifier
 
 def contains_left_button(buttons) -> bool:
     """
@@ -2170,14 +2327,13 @@ def get_char() -> str:
     return _win.get_char()
 
 
-def get_key() -> (int, int):
+def get_key() -> KeyMessage:
     """
     Get the key inputted by keyboard.
 
     If not any  key is pressed in last 100 ms, the program will stop and wait for the next key hitting.
 
-    :return: `keyboard code <http://pyqt.sourceforge.net/Docs/PyQt4/qt.html#Key-enum/>`_ ,
-        `keyboard modifier codes <http://pyqt.sourceforge.net/Docs/PyQt4/qt.html#KeyboardModifier-enum)/>`_
+    :return: key message
     """
     _check_not_headless_and_in_shell()
     return _win.get_key()
@@ -2195,6 +2351,7 @@ def get_cursor_pos() -> (int, int):
 
 # init and close graphics #
 
+@invoke_in_app_thread.invoke_in_thread()
 def set_caption(title: str):
     """
     Set the graph window\'s caption
@@ -2203,7 +2360,11 @@ def set_caption(title: str):
     """
     _win.setWindowTitle(title)
 
+
 @invoke_in_app_thread.invoke_in_thread()
+def _init_graph_in_thread(width:int ,height:int,headless:bool):
+    _init_graph(width,height,headless)
+
 def _init_graph(width:int ,height:int,headless:bool):
     """
     Init the graphics context
@@ -2217,7 +2378,7 @@ def _init_graph(width:int ,height:int,headless:bool):
         _win = GraphWin(width, height)
         _target_image = _win.get_canvas()
         _win.show()
-        set_caption("Python Easy Graphics")
+        _win.setWindowTitle("Python Easy Graphics")
         set_font_size(18)
     if _in_shell:
         _get_target_image = _get_target_image_in_shell
@@ -2243,7 +2404,7 @@ def init_graph(width: int = 800, height: int = 600, headless: bool = False):
     if _is_run:
         raise RuntimeError("The Graphics Windows is already inited!")
     if _easy_run_mode:
-        _init_graph(width,height,headless)
+        _init_graph_in_thread(width,height,headless)
         return
 
     # prepare Events
@@ -2341,8 +2502,7 @@ def add_record(image: Image = None, **options):
     """
     Add one frame to the recording animation
 
-    :param image: the target image whose content will be captured. None means it is the target image
-    (see set_target() and get_target()).
+    :param image: the target image whose content will be captured. None means it is the target image (see set_target() and get_target()).
     """
     if _animation is None:
         raise RuntimeError("There's no animation in recording!")
