@@ -1,6 +1,7 @@
 import os
 import threading
 import time
+import queue
 from PyQt5 import QtWidgets
 from PyQt5 import QtCore
 from PyQt5 import QtGui
@@ -31,13 +32,12 @@ class GraphWin(QtWidgets.QWidget):
         super().__init__(flags=QtCore.Qt.Window | QtCore.Qt.MSWindowsFixedSizeDialogHint)
         self._width = width
         self._height = height
+        self.setFixedWidth(width)
+        self.setFixedHeight(height)
         self._wait_event = threading.Event()
-        self._mouse_event = threading.Event()
-        self._key_event = threading.Event()
-        self._char_key_event = threading.Event()
-        self._key_msg = _KeyMsg()
-        self._key_char_msg = _KeyCharMsg()
-        self._mouse_msg = _MouseMsg()
+        self._key_msg_queue = queue.Queue(10)
+        self._key_char_msg_queue = queue.Queue(10)
+        self._mouse_msg_queue = queue.Queue(10)
         self.setGeometry(100, 100, width, height)
         self._init_screen(width, height)
         self._is_run = True
@@ -103,8 +103,13 @@ class GraphWin(QtWidgets.QWidget):
             self._canvas.remove_updated_listener(self.update)
 
     def close(self):
+        self._key_msg_queue.queue.clear()
+        self._key_char_msg_queue.queue.clear()
+        self._mouse_msg_queue.queue.clear()
         if self._immediate:
             self._canvas.remove_updated_listener(self.update)
+        self._is_run = False
+        self._wait_event.set()
 
     def is_immediate(self) -> bool:
         """
@@ -116,12 +121,25 @@ class GraphWin(QtWidgets.QWidget):
 
     def mousePressEvent(self, e: QtGui.QMouseEvent):
         self._wait_event.set()
-        self._mouse_msg.set_event(e, MouseMessageType.PRESS_MESSAGE)
-        self._mouse_event.set()
+        mouse_msg=MouseMessage(e, MouseMessageType.PRESS_MESSAGE)
+        try:
+            self._mouse_msg_queue.put_nowait(mouse_msg)
+        except queue.Full as err:
+            pass
+
+    def mouseDoubleClickEvent(self, e: QtGui.QMouseEvent) -> None:
+        mouse_msg=MouseMessage(e, MouseMessageType.DOUBLE_CLICK_MESSAGE)
+        try:
+            self._mouse_msg_queue.put_nowait(mouse_msg)
+        except queue.Full as err:
+            pass
 
     def mouseReleaseEvent(self, e: QtGui.QMouseEvent):
-        self._mouse_msg.set_event(e, MouseMessageType.RELEASE_MESSAGE)
-        self._mouse_event.set()
+        mouse_msg=MouseMessage(e, MouseMessageType.RELEASE_MESSAGE)
+        try:
+            self._mouse_msg_queue.put_nowait(mouse_msg)
+        except queue.Full as err:
+            pass
 
     def keyPressEvent(self, e: QtGui.QKeyEvent):
         self._wait_event.set()
@@ -134,10 +152,16 @@ class GraphWin(QtWidgets.QWidget):
                 self._canvas.save(self._capture_dir + os.sep + "save{0}.png".format(self._capture_count))
         if e.key() < 127 or e.key() == QtCore.Qt.Key_Return:
             # ascii char key pressed
-            self._key_char_msg.set_char(e)
-            self._char_key_event.set()
-        self._key_msg.set_event(e)
-        self._key_event.set()
+            key_char_msg = _KeyCharMsg(e)
+            try:
+                self._key_char_msg_queue.put_nowait(key_char_msg)
+            except queue.Full as err:
+                pass
+        key_msg = KeyMessage(e)
+        try:
+            self._key_msg_queue.put_nowait(key_msg)
+        except queue.Full as err:
+            pass
 
     def pause(self):
         """
@@ -150,11 +174,7 @@ class GraphWin(QtWidgets.QWidget):
         self._wait_event.wait()
 
     def closeEvent(self, e: QtGui.QCloseEvent):
-        self._is_run = False
-        self._wait_event.set()
-        self._mouse_event.set()
-        self._key_event.set()
-        self._char_key_event.set()
+        self.close()
 
     def is_run(self) -> bool:
         return self._is_run
@@ -200,6 +220,7 @@ class GraphWin(QtWidgets.QWidget):
             self._last_fps_time = time.perf_counter_ns()
         self.real_update()
         tt = time.perf_counter_ns()
+        # print(tt)
         if tt - self._last_fps_time < nanotime:
             QtCore.QThread.usleep((self._last_fps_time + nanotime - tt) // 1000)
         self._last_fps_time = time.perf_counter_ns()
@@ -263,94 +284,105 @@ class GraphWin(QtWidgets.QWidget):
             return ' '
         nt = time.perf_counter_ns()
         self.real_update()
-        if nt - self._key_char_msg.get_time() > 100000000:
-            # if the last char msg is 100ms ago, we wait for a new msg
-            self._char_key_event.clear()
-            self._char_key_event.wait()
-        ch = self._key_char_msg.get_char()
-        self._key_char_msg.reset()
-        return ch
+        while True:
+            try:
+                key_char_msg = self._key_char_msg_queue.get(True,0.1)
+                return key_char_msg.char
+            except queue.Empty as err:
+                if not self._is_run:
+                    return ''
+                pass
 
-    def get_key(self) -> (int, int):
+
+
+    def get_key(self) -> "KeyMessage":
         """
         Get the key inputted by keyboard.
 
         If not any  key is pressed in last 100 ms, the program will stop and wait for the next key hitting.
 
-        :return: `keyboard code <http://pyqt.sourceforge.net/Docs/PyQt4/qt.html#Key-enum/>`_ ,
-            `keyboard modifier codes <http://pyqt.sourceforge.net/Docs/PyQt4/qt.html#KeyboardModifier-enum)/>`_
+        :return: key message
         """
         if not self._is_run:
             return QtCore.Qt.Key_Escape, QtCore.Qt.NoModifier
         nt = time.perf_counter_ns()
         self.real_update()
-        if nt - self._key_msg.get_time() > 100000000:
-            # if the last key msg is 100ms ago, we wait for a new msg
-            self._key_event.clear()
-            self._key_event.wait()
-        e = self._key_msg.get_event()
-        if e is None:
-            return QtCore.Qt.Key_Escape, QtCore.Qt.NoModifier
-        self._key_msg.reset()
-        return e.key(), e.modifiers()
+        while True:
+            try:
+                key_msg = self._key_msg_queue.get(True,0.1)
+                return key_msg
+            except queue.Empty as err:
+                if not self._is_run:
+                    return KeyMessage(None)
+                pass
 
-    def get_mouse_msg(self) -> (int, int, int, int):
+
+
+    def get_mouse_msg(self) -> "MouseMessage":
         """
         Get the mouse message.
 
-        If there is not any  mouse button is pressed or released in last 100 ms, the program will stop and wait for
-        the next mouse message.
+        If there is not any  mouse message, the program will stop and wait for it.
 
-        :return: x of the cursor, y of the cursor , type, mouse buttons down
-            ( QtCore.Qt.LeftButton or QtCore.Qt.RightButton or QtCore.Qt.MidButton or QtCore.Qt.NoButton)
+        :return: mouse message
         """
         if not self._is_run:
             return 0, 0, 0, QtCore.Qt.NoButton
         nt = time.perf_counter_ns()
         self.real_update()
-        if nt - self._mouse_msg.get_time() > 100000000:
-            # if the last key msg is 100ms ago, we wait for a new msg
-            self._mouse_event.clear()
-            self._mouse_event.wait()
-        e = self._mouse_msg.get_event()
-        if e is None:
-            return 0, 0, 0, QtCore.Qt.NoButton
-        _type = self._mouse_msg.get_type()
-        self._mouse_msg.reset()
-        return e.x(), e.y(), _type, e.button()
+        while True:
+            try:
+                mouse_msg = self._mouse_msg_queue.get(True,0.1)
+                return mouse_msg
+            except queue.Empty as err:
+                if not self._is_run:
+                    return MouseMessage(None,None)
+                pass
+
+    def get_click(self) -> "MouseMessage":
+        """
+        Get the mouse click message.
+
+        If there isn't any clicked event, the program will stop and wait for it.
+
+        :return: mouse message
+        """
+        while self._is_run:
+            msg = self.get_mouse_msg()
+            if msg == MouseMessageType.NO_MESSAGE or\
+                    msg.type == MouseMessageType.RELEASE_MESSAGE and (msg.button & QtCore.Qt.LeftButton) > 0:
+                return msg
+        return MouseMessage(None,None)
 
     def has_kb_hit(self) -> bool:
         """
-        See if any ascii char key is hitted in the last 100 ms.
+        See if any ascii char key hit message in the message queue.
 
         Use it with get_char().
 
         :return:  True if hit, False otherwise
         """
-        nt = time.perf_counter_ns()
-        return nt - self._key_char_msg.get_time() <= 100000000
+        return not self._key_char_msg_queue.empty()
 
     def has_kb_msg(self) -> bool:
         """
-        See if any key is hit in the last 100 ms.
+        See if any key hit message in the message queue.
 
         Use it with get_key().
 
         :return:  True if hit, False otherwise
         """
-        nt = time.perf_counter_ns()
-        return nt - self._key_char_msg.get_time() <= 100000000
+        return not self._key_msg_queue.empty()
 
     def has_mouse_msg(self) -> bool:
         """
-        See if there is any mouse message(event) in the last 100 ms.
+        See if there is any mouse message(event) in the message queue.
 
         Use it with get_mouse_msg().
 
         :return:  True if any mouse message, False otherwise
         """
-        nt = time.perf_counter_ns()
-        return nt - self._mouse_msg.get_time() <= 100000000
+        return not self._mouse_msg_queue.empty()
 
     def get_cursor_pos(self) -> (int, int):
         """
@@ -361,29 +393,40 @@ class GraphWin(QtWidgets.QWidget):
         p = self.mapFromGlobal(QtGui.QCursor.pos())
         return p.x(), p.y()
 
+    def clear_key_msg(self):
+        """
+        Clear all keyboard hit messages.
+        """
+        self._key_msg_queue.queue.clear()
 
-class _KeyMsg:
+    def clear_char_msg(self):
+        """
+        Clear all char key hit messages.
+        """
+        self._key_char_msg_queue.queue.clear()
+
+    def clear_mouse_msg(self):
+        """
+        Clear all mouse messages.
+        """
+        self._mouse_msg_queue.queue.clear()
+
+class KeyMessage:
     """
     class for saving keyboard message
     """
 
-    def __init__(self):
-        self._time = 0
-        self._key_event = None
-
-    def set_event(self, key_event: QtGui.QKeyEvent):
-        self._key_event = key_event
-        self._time = time.perf_counter_ns()
-
-    def get_event(self) -> QtGui.QKeyEvent:
-        return self._key_event
-
-    def get_time(self) -> int:
-        return self._time
-
-    def reset(self):
-        self._time = 0
-        self._key_event = None
+    def __init__(self,key_event: QtGui.QKeyEvent):
+        if key_event is None:
+            self.key = None
+            self.char = ''
+            self.count = 0
+            self.modifiers = QtCore.Qt.NoModifier
+        else:
+            self.key = key_event.key()
+            self.char = key_event.text()
+            self.count = key_event.count()
+            self.modifiers = key_event.modifiers()
 
 
 class _KeyCharMsg:
@@ -391,47 +434,24 @@ class _KeyCharMsg:
     class for saving keyboard hit char
     """
 
-    def __init__(self):
-        self._time = 0
-        self._key = None
+    def __init__(self, key_event: QtGui.QKeyEvent):
+        self.char = key_event.text()
 
-    def set_char(self, key_event: QtGui.QKeyEvent):
-        key_event.key()
-        self._key = key_event.text()
-        self._time = time.perf_counter_ns()
-
-    def get_char(self) -> str:
-        return self._key
-
-    def get_time(self):
-        return self._time
-
-    def reset(self):
-        self._time = 0
-        self._key = None
-
-
-class _MouseMsg:
-    def __init__(self):
-        self._time = 0
-        self._mouse_event = None
-        self._type = MouseMessageType.NO_MESSAGE
-
-    def set_event(self, e: QtGui.QMouseEvent, _type: int):
-        self._mouse_event = e
-        self._time = time.perf_counter_ns()
-        self._type = _type
-
-    def get_event(self) -> QtGui.QMouseEvent:
-        return self._mouse_event
-
-    def get_time(self):
-        return self._time
-
-    def get_type(self):
-        return self._type
-
-    def reset(self):
-        self._time = 0
-        self._mouse_event = None
-        self._type = MouseMessageType.NO_MESSAGE
+class MouseMessage:
+    def __init__(self, e: QtGui.QMouseEvent, _type: MouseMessageType):
+        if e is None:
+            self.x = 0
+            self.y = 0
+            self.modifiers = QtCore.Qt.NoModifier
+            self.global_x = 0
+            self.global_y = 0
+            self.type = MouseMessageType.NO_MESSAGE
+            self.button = QtCore.Qt.NoButton
+        else:
+            self.x = e.x()
+            self.y = e.y()
+            self.modifiers = e.modifiers()
+            self.global_x = e.globalX()
+            self.global_y = e.globalY()
+            self.type = _type
+            self.button = e.button()
