@@ -1,4 +1,5 @@
 from collections import deque
+from pathlib import Path
 from typing import Union, Callable
 import math
 
@@ -84,6 +85,14 @@ class Image:
         # p.setRenderHint(QtGui.QPainter.Antialiasing) # flood fill will not work when anti-aliasing is on
         self._default_rect = p.viewport()
 
+    def set_antialiasing(self,anti:bool=True):
+        """
+        Set Anti Aliasing
+
+        :param anti: if antialiasing should be set
+        """
+        self._painter.setRenderHint(QtGui.QPainter.Antialiasing,anti)
+
     def _init_mask_painter(self):
         p = self._mask_painter
         p.begin(self._mask)
@@ -541,6 +550,7 @@ class Image:
         """
         Reset all transforms (translate/rotate/scale).
         """
+        self._flip_y = False
         self._painter.resetTransform()
         self._mask_painter.resetTransform()
 
@@ -1297,8 +1307,8 @@ class Image:
         self.fill_rect(-1, -1, self.get_width() + 2, self.get_height() + 2)
         self.restore_settings()
 
-    def draw_image(self, x: int, y: int, image: "Image", src_x: int = 0, src_y: int = 0, src_width: int = -1,
-                   src_height: int = -1, with_background=True, composition_mode=None):
+    def draw_image(self, x: int, y: int, image: "Image", src_x: int = 0, src_y: int = 0, src_width: int = 0,
+                   src_height: int = 0, with_background=True, composition_mode=None):
         """
         Copy part of the source image (src_image) to the destination image (dst_image).
 
@@ -1826,6 +1836,16 @@ class Image:
         buffer.close()
         return ba.data()
 
+    def scaled(self,width:int,height:int) -> "Image":
+        """
+        Create a scaled copy with the specified width and height.
+        :param width: width of the copy
+        :param height:  height of the copy
+        :return: new copy
+        """
+        new_image =  self._image.scaled(width,height,QtCore.Qt.IgnoreAspectRatio, QtCore.Qt.SmoothTransformation)
+        return Image(new_image)
+
     def set_rect_mode(self, mode):
         self._rect_mode = mode
 
@@ -1891,6 +1911,9 @@ class Image:
         :param filename: the image file
         :return: the loaded image
         """
+        p = Path(filename)
+        if not p.exists():
+            raise ValueError(f"File {filename} not exists.")
         image = QtGui.QImage(filename)
         return Image(image)
 
